@@ -16,18 +16,18 @@ class ConfLine {
 	char *section;
 	char *norm_var;
 
-	void _set(char **dst, const char *val);
+	void _set(char **dst, const char *val, bool alloc);
 public:
 	ConfLine() : prefix(NULL), var(NULL), mid(NULL), val(NULL),
 		   suffix(NULL), section(NULL), norm_var(NULL) {}
 	~ConfLine();
 
-	void set_prefix(const char *val);
-	void set_var(const char *val);
-	void set_mid(const char *val);
-	void set_val(const char *val);
-	void set_suffix(const char *val);
-	void set_section(const char *val);
+	void set_prefix(const char *val, bool alloc = true);
+	void set_var(const char *val, bool alloc = true);
+	void set_mid(const char *val, bool alloc = true);
+	void set_val(const char *val, bool alloc = true);
+	void set_suffix(const char *val, bool alloc = true);
+	void set_section(const char *val, bool alloc = true);
 
 	char *get_prefix() { return prefix; }
 	char *get_var() { return var; }
@@ -83,16 +83,16 @@ class ConfFile {
 
 	ConfSection *_add_section(const char *section, ConfLine *cl);
 	void _dump(int fd);
-	int _parse(char *filename, ConfSection **psection);
+	bool _parse(char *filename, ConfSection **psection);
 public:
-        ConfFile(const char *fname) : filename(strdup(fname)), auto_update(false),
-				      post_process_func(NULL) {}
+        ConfFile(const char *fname) : auto_update(false),
+				      post_process_func(NULL) { if (fname) filename = strdup(fname); }
 	~ConfFile();
 
 	const SectionList& get_section_list() { return sections_list; }
 	const char *get_filename() { return filename; }
 
-	int parse();
+	bool parse();
 	int read(const char *section, const char *var, int *val, int def_val);
 	int read(const char *section, const char *var, unsigned int *val, unsigned int def_val);
 	int read(const char *section, const char *var, long long *val, long long def_val);
