@@ -84,8 +84,9 @@ public:
     version_t state_seq;
     entity_addr_t addr;
     utime_t laggy_since;
-    int standby_for_rank;
+    int32_t standby_for_rank;
     string standby_for_name;
+    set<int32_t> export_targets;
 
     mds_info_t() : rank(-1), inc(0), state(STATE_STANDBY), state_seq(0) { }
 
@@ -95,7 +96,7 @@ public:
     entity_inst_t get_inst() const { return entity_inst_t(entity_name_t::MDS(rank), addr); }
 
     void encode(bufferlist& bl) const {
-      __u8 v = 1;
+      __u8 v = 2;
       ::encode(v, bl);
       ::encode(name, bl);
       ::encode(rank, bl);
@@ -106,6 +107,7 @@ public:
       ::encode(laggy_since, bl);
       ::encode(standby_for_rank, bl);
       ::encode(standby_for_name, bl);
+      ::encode(export_targets, bl);
     }
     void decode(bufferlist::iterator& bl) {
       __u8 v;
@@ -119,6 +121,8 @@ public:
       ::decode(laggy_since, bl);
       ::decode(standby_for_rank, bl);
       ::decode(standby_for_name, bl);
+      if (v >= 2)
+	::decode(export_targets, bl);
     }
   };
 
@@ -172,6 +176,7 @@ public:
   utime_t get_session_timeout() {
     return utime_t(session_timeout,0);
   }
+  __u64 get_max_filesize() { return max_file_size; }
   
   epoch_t get_epoch() const { return epoch; }
   void inc_epoch() { epoch++; }
@@ -195,6 +200,14 @@ public:
   __u32 get_metadata_pg_pool() const { return metadata_pg_pool; }
 
   const map<entity_addr_t,mds_info_t>& get_mds_info() { return mds_info; }
+  const mds_info_t& get_mds_info(entity_addr_t a) {
+    assert(mds_info.count(a));
+    return mds_info[a];
+  }
+  const mds_info_t& get_mds_info(int m) {
+    assert(up.count(m) && mds_info.count(up[m]));
+    return mds_info[up[m]];
+  }
 
   // counts
   unsigned get_num_mds() {
@@ -350,6 +363,10 @@ public:
     assert(up.count(m));
     return mds_info[up[m]].get_inst();
   }
+  const entity_addr_t get_addr(int m) {
+    assert(up.count(m));
+    return mds_info[up[m]].addr;
+  }
   bool get_inst(int m, entity_inst_t& inst) { 
     if (up.count(m)) {
       inst = get_inst(m);
