@@ -16,29 +16,31 @@
 #ifndef __MGETPOOLSTATSREPLY_H
 #define __MGETPOOLSTATSREPLY_H
 
-class MGetPoolStatsReply : public Message {
+class MGetPoolStatsReply : public PaxosServiceMessage {
 public:
   ceph_fsid_t fsid;
   tid_t tid;
   map<string,pool_stat_t> pool_stats;
 
-  MGetPoolStatsReply() : Message(MSG_GETPOOLSTATSREPLY) {}
-  MGetPoolStatsReply(ceph_fsid_t& f, tid_t t) :
-    Message(MSG_GETPOOLSTATSREPLY),
+  MGetPoolStatsReply() : PaxosServiceMessage(MSG_GETPOOLSTATSREPLY, 0) {}
+  MGetPoolStatsReply(ceph_fsid_t& f, tid_t t, version_t v) :
+    PaxosServiceMessage(MSG_GETPOOLSTATSREPLY, v),
     fsid(f), tid(t) { }
 
   const char *get_type_name() { return "getpoolstats"; }
   void print(ostream& out) {
-    out << "getpoolstatsreply(" << tid << ")";
+    out << "getpoolstatsreply(" << tid << " v" << version <<  ")";
   }
 
   void encode_payload() {
+    paxos_encode();
     ::encode(fsid, payload);
     ::encode(tid, payload);
     ::encode(pool_stats, payload);
   }
   void decode_payload() {
     bufferlist::iterator p = payload.begin();
+    paxos_decode(p);
     ::decode(fsid, p);
     ::decode(tid, p);
     ::decode(pool_stats, p);
