@@ -47,7 +47,8 @@ int MonClient::build_initial_monmap()
     int r = monmap.read(monmap_fn);
     if (r >= 0)
       return 0;
-    cerr << "unable to read monmap from " << monmap_fn << ": " << strerror(errno) << std::endl;
+    char buf[80];
+    cerr << "unable to read monmap from " << monmap_fn << ": " << strerror_r(errno, buf, sizeof(buf)) << std::endl;
   }
 
   // -m foo?
@@ -102,11 +103,29 @@ int MonClient::build_initial_monmap()
   return -ENOENT;
 }
 
+
 int MonClient::get_monmap()
 {
   dout(10) << "get_monmap" << dendl;
   Mutex::Locker l(monc_lock);
   
+  want_monmap = true;
+  if (cur_mon < 0)
+    _reopen_session();
+
+  while (want_monmap)
+    map_cond.Wait(monc_lock);
+
+  dout(10) << "get_monmap done" << dendl;
+
+  return 0;
+}
+
+int MonClient::get_monmap_privately()
+{
+  dout(10) << "get_monmap_privately" << dendl;
+  Mutex::Locker l(monc_lock);
+  
   SimpleMessenger *rank = NULL; 
   bool temp_msgr = false;
   if (!messenger) {
@@ -114,17 +133,17 @@ int MonClient::get_monmap()
     rank->bind();
     rank->set_policy(entity_name_t::TYPE_MON, SimpleMessenger::Policy::lossy_fast_fail());
     messenger = rank->register_entity(entity_name_t::CLIENT(-1));
-    messenger->set_dispatcher(this);
+    messenger->add_dispatcher_head(this);
     rank->start(true);  // do not daemonize!
     temp_msgr = true; 
   }
   
   int attempt = 10;
   int i = 0;
-
   srand(getpid());
+  
   dout(10) << "have " << monmap.epoch << dendl;
-    
+  
   while (monmap.epoch == 0) {
     i = rand() % monmap.mon_inst.size();
     dout(10) << "querying " << monmap.mon_inst[i] << dendl;
@@ -143,7 +162,7 @@ int MonClient::get_monmap()
     messenger->destroy();
     messenger = 0;
   }
-
+ 
   if (monmap.epoch)
     return 0;
   return -1;
@@ -179,9 +198,16 @@ void MonClient::handle_monmap(MMonMap *m)
   dout(10) << "handle_monmap " << *m << dendl;
   monc_lock.Lock();
 
+  _finish_hunting();
+
   bufferlist::iterator p = m->monmapbl.begin();
   ::decode(monmap, p);
+
+  _sub_got("monmap", monmap.get_epoch());
+
   map_cond.Signal();
+  want_monmap = false;
+
   monc_lock.Unlock();
   delete m;
 }
@@ -201,7 +227,7 @@ void MonClient::_send_mount()
 void MonClient::init()
 {
   dout(10) << "init" << dendl;
-  messenger->set_dispatcher(this);
+  messenger->add_dispatcher_head(this);
 
   Mutex::Locker l(monc_lock);
   timer.add_event_after(10.0, new C_Tick(this));
@@ -222,15 +248,21 @@ int MonClient::mount(double mount_timeout)
   }
 
   // only first mounter does the work
-  if (!mounting)
-    _send_mount();
   mounting++;
+  if (mounting == 1) {
+    if (cur_mon < 0)
+      _reopen_session();
+    else
+      _send_mount();
+  }
   while (clientid < 0 && !mount_err)
     mount_cond.Wait(monc_lock);
   mounting--;
 
   if (clientid >= 0) {
     dout(5) << "mount success, client" << clientid << dendl;
+
+    _sub_want("monmap", monmap.get_epoch());
   }
 
   return mount_err;
@@ -240,7 +272,7 @@ void MonClient::handle_mount_ack(MClientMountAck* m)
 {
   dout(10) << "handle_mount_ack " << *m << dendl;
 
-  hunting = false;
+  _finish_hunting();
 
   // monmap
   bufferlist::iterator p = m->monmap_bl.begin();
@@ -259,28 +291,49 @@ void MonClient::handle_mount_ack(MClientMountAck* m)
 
 void MonClient::_send_mon_message(Message *m)
 {
-  int mon = monmap.pick_mon();
-  messenger->send_message(m, monmap.mon_inst[mon]);
+  assert(cur_mon >= 0);
+  messenger->send_message(m, monmap.mon_inst[cur_mon]);
 }
 
 void MonClient::_pick_new_mon()
 {
-  int oldmon = monmap.pick_mon();
-  messenger->mark_down(monmap.get_inst(oldmon).addr);
-  monmap.pick_mon(true);
+  if (cur_mon >= 0)
+    messenger->mark_down(monmap.get_inst(cur_mon).addr);
+  cur_mon = monmap.pick_mon(true);
+  dout(10) << "_pick_new_mon picked mon" << cur_mon << dendl;
 }
 
 
-void MonClient::ms_handle_reset(const entity_addr_t& peer)
+void MonClient::_reopen_session()
+{
+  dout(10) << "_reopen_session" << dendl;
+  _pick_new_mon();
+  if (mounting)
+    _send_mount();
+  if (!sub_have.empty())
+    _renew_subs();
+  if (!mounting && sub_have.empty()) {
+    _send_mon_message(new MMonGetMap);
+  }
+}
+
+bool MonClient::ms_handle_reset(Connection *con, const entity_addr_t& peer)
 {
   dout(10) << "ms_handle_reset " << peer << dendl;
-  if (!hunting) {
-    dout(0) << "staring hunt for new mon" << dendl;
-    hunting = true;
-    _pick_new_mon();
-    if (mounting)
-      _send_mount();
-    _renew_subs();
+  if (hunting)
+    return true;
+
+  dout(0) << "hunting for new mon" << dendl;
+  hunting = true;
+  _reopen_session();
+  return false;
+}
+
+void MonClient::_finish_hunting()
+{
+  if (hunting) {
+    dout(0) << "found mon" << cur_mon << dendl; 
+    hunting = false;
   }
 }
 
@@ -290,11 +343,7 @@ void MonClient::tick()
 
   if (hunting) {
     dout(0) << "continuing hunt" << dendl;
-    // try new monitor
-    _pick_new_mon();
-    if (mounting)
-      _send_mount();
-    _renew_subs();
+    _reopen_session();
   } else {
     // just renew as needed
     utime_t now = g_clock.now();
@@ -306,8 +355,6 @@ void MonClient::tick()
   }
 
   timer.add_event_after(10.0, new C_Tick(this));
-  dout(10) << "tick done" << dendl;
-
 }
 
 
@@ -321,18 +368,21 @@ void MonClient::_renew_subs()
   }
 
   dout(10) << "renew_subs" << dendl;
+  if (cur_mon < 0)
+    _reopen_session();
+  else {
+    if (sub_renew_sent == utime_t())
+      sub_renew_sent = g_clock.now();
 
-  if (sub_renew_sent == utime_t())
-    sub_renew_sent = g_clock.now();
-
-  MMonSubscribe *m = new MMonSubscribe;
-  m->what = sub_have;
-  _send_mon_message(m);
+    MMonSubscribe *m = new MMonSubscribe;
+    m->what = sub_have;
+    _send_mon_message(m);
+  }
 }
 
 void MonClient::handle_subscribe_ack(MMonSubscribeAck *m)
 {
-  hunting = false;
+  _finish_hunting();
 
   if (sub_renew_sent != utime_t()) {
     sub_renew_after = sub_renew_sent;
