@@ -53,7 +53,7 @@ e 12v
 #include "include/types.h"
 #include "mon_types.h"
 #include "include/buffer.h"
-#include "msg/Message.h"
+#include "messages/PaxosServiceMessage.h"
 #include "msg/msg_types.h"
 
 #include "include/Context.h"
@@ -240,7 +240,7 @@ public:
     return machine_name;
   }
 
-  void dispatch(Message *m);
+  void dispatch(PaxosServiceMessage *m);
 
   void init();
 
@@ -258,11 +258,11 @@ public:
   
   // read
   version_t get_version() { return last_committed; }
-  bool is_readable();
+  bool is_readable(version_t seen=0);
   bool read(version_t v, bufferlist &bl);
   version_t read_current(bufferlist &bl);
   void wait_for_readable(Context *onreadable) {
-    assert(!is_readable());
+    //assert(!is_readable());
     waiting_for_readable.push_back(onreadable);
   }
 
