@@ -86,14 +86,16 @@
 #define MSG_MDS_CACHEREJOIN        0x202
 #define MSG_MDS_DISCOVER           0x203
 #define MSG_MDS_DISCOVERREPLY      0x204
-#define MSG_MDS_INODEUPDATE  0x205
-#define MSG_MDS_DIRUPDATE    0x206
-#define MSG_MDS_CACHEEXPIRE  0x207
-#define MSG_MDS_DENTRYUNLINK      0x208
-#define MSG_MDS_FRAGMENTNOTIFY 0x209
+#define MSG_MDS_INODEUPDATE        0x205
+#define MSG_MDS_DIRUPDATE          0x206
+#define MSG_MDS_CACHEEXPIRE        0x207
+#define MSG_MDS_DENTRYUNLINK       0x208
+#define MSG_MDS_FRAGMENTNOTIFY     0x209
+#define MSG_MDS_OFFLOAD_TARGETS    0x20a
+#define MSG_MDS_OFFLOAD_COMPLETE   0x20b
 
-#define MSG_MDS_LOCK             0x300
-#define MSG_MDS_INODEFILECAPS    0x301
+#define MSG_MDS_LOCK               0x300
+#define MSG_MDS_INODEFILECAPS      0x301
 
 #define MSG_MDS_EXPORTDIRDISCOVER     0x449
 #define MSG_MDS_EXPORTDIRDISCOVERACK  0x450
