@@ -368,13 +368,20 @@ private:
   bool is_out(int osd) { return !exists(osd) || get_weight(osd) == CEPH_OSD_OUT; }
   bool is_in(int osd) { return exists(osd) && !is_out(osd); }
   
+  int identify_osd(const entity_addr_t& addr) const {
+    for (unsigned i=0; i<osd_addr.size(); i++)
+      if (osd_addr[i] == addr)
+	return i;
+    return -1;
+  }
   bool have_addr(const entity_addr_t& addr) const {
-    for (vector<entity_addr_t>::const_iterator p = osd_addr.begin();
-	 p != osd_addr.end();
-	 p++)
-      if (*p == addr)
-	return true;
-    return false;
+    return identify_osd(addr) >= 0;
+  }
+  bool find_osd_on_ip(const entity_addr_t& ip) const {
+    for (unsigned i=0; i<osd_addr.size(); i++)
+      if (osd_addr[i].is_same_host(ip))
+	return i;
+    return -1;
   }
   bool have_inst(int osd) {
     return exists(osd) && is_up(osd); 
@@ -400,7 +407,7 @@ private:
     assert(exists(osd));
     entity_inst_t i(entity_name_t::OSD(osd),
 		    osd_addr[osd]);
-    i.addr.erank++;  // heartbeat addr erank is regular addr erank + 1
+    i.addr.v.erank = i.addr.v.erank + 1;  // heartbeat addr erank is regular addr erank + 1
     return i;
   }
 
@@ -624,8 +631,8 @@ private:
   // oid -> pg
   ceph_object_layout file_to_object_layout(object_t oid, ceph_file_layout& layout) {
     return make_object_layout(oid, layout.fl_pg_pool,
-			      ceph_file_layout_pg_preferred(layout),
-			      ceph_file_layout_object_su(layout));
+			      layout.fl_pg_preferred,
+			      layout.fl_stripe_unit);
   }
 
   ceph_object_layout make_object_layout(object_t oid, int pg_pool, int preferred=-1, int object_stripe_unit = 0) {
