@@ -24,7 +24,7 @@
 
 
 
-#define CEPH_OSD_ONDISK_MAGIC "ceph osd volume v022"
+#define CEPH_OSD_ONDISK_MAGIC "ceph osd volume v023"
 
 
 
@@ -130,22 +130,32 @@ struct pg_t {
   
   operator uint64_t() const { return u.pg64; }
 
-  coll_t to_coll() const {
+  /*coll_t to_coll() const {
     return coll_t(u.pg64, 0); 
   }
   coll_t to_snap_coll(snapid_t sn) const {
     return coll_t(u.pg64, sn);
-  }
+    }*/
 
+  int print(char *o) {
+    if (preferred() >= 0)
+      return sprintf(o, "%d.%xp%d", pool(), ps(), preferred());
+    else
+      return sprintf(o, "%d.%x", pool(), ps());
+  }
   bool parse(const char *s) {
     int pool;
     int ps;
-    int r = sscanf(s, "%d.%x", &pool, &ps);
+    int preferred;
+    int r = sscanf(s, "%d.%xp%x", &pool, &ps, &preferred);
     if (r < 2)
       return false;
     u.pg.pool = pool;
     u.pg.ps = ps;
-    u.pg.preferred = -1;
+    if (r == 3)
+      u.pg.preferred = preferred;
+    else
+      u.pg.preferred = -1;
     return true;
   }
 
@@ -183,6 +193,94 @@ namespace __gnu_cxx {
 }
 
 
+// ----------------------
+
+struct coll_t {
+  pg_t pgid;
+  snapid_t snap;
+
+  coll_t(__u64 p=0, snapid_t s=0) : pgid(p), snap(s) {}
+  
+  static coll_t build_pg_coll(pg_t p) {
+    return coll_t(p, CEPH_NOSNAP);
+  }
+  static coll_t build_snap_pg_coll(pg_t p, snapid_t s) {
+    return coll_t(p, s);
+  }
+
+  int print(char *o) {
+    if (pgid == pg_t() && snap == 0)
+      return sprintf(o, "meta");
+    int len = pgid.print(o);
+    if (snap != CEPH_NOSNAP)
+      len += sprintf(o+len, "_%llx", (long long unsigned)snap);
+    else {
+      strcat(o+len, "_head");
+      len += 5;
+    }
+    return len;
+  }
+  bool parse(char *s) {
+    if (strncmp(s, "meta", 4) == 0) {
+      *this = coll_t();
+      return true;
+    }
+    if (!pgid.parse(s))
+      return false;
+    char *sn = strchr(s, '_');
+    if (!sn)
+      return false;
+    if (strncmp(sn, "_head", 5) == 0)
+      snap = CEPH_NOSNAP;
+    else
+      snap = strtoull(sn+1, 0, 16);
+    return true;
+  }
+
+  void encode(bufferlist& bl) const {
+    ::encode(pgid, bl);
+    ::encode(snap, bl);
+  }
+  void decode(bufferlist::iterator& bl) {
+    ::decode(pgid, bl);
+    ::decode(snap, bl);
+  }
+};
+WRITE_CLASS_ENCODER(coll_t)
+
+inline ostream& operator<<(ostream& out, const coll_t& c) {
+  return out << hex << c.pgid << '_' << c.snap << dec;
+}
+
+inline bool operator<(const coll_t& l, const coll_t& r) {
+  return l.pgid < r.pgid || (l.pgid == r.pgid && l.snap < r.snap);
+}
+inline bool operator<=(const coll_t& l, const coll_t& r) {
+  return l.pgid < r.pgid || (l.pgid == r.pgid && l.snap <= r.snap);
+}
+inline bool operator==(const coll_t& l, const coll_t& r) {
+  return l.pgid == r.pgid && l.snap == r.snap;
+}
+inline bool operator!=(const coll_t& l, const coll_t& r) {
+  return l.pgid != r.pgid || l.snap != r.snap;
+}
+inline bool operator>(const coll_t& l, const coll_t& r) {
+  return l.pgid > r.pgid || (l.pgid == r.pgid && l.snap > r.snap);
+}
+inline bool operator>=(const coll_t& l, const coll_t& r) {
+  return l.pgid > r.pgid || (l.pgid == r.pgid && l.snap >= r.snap);
+}
+
+namespace __gnu_cxx {
+  template<> struct hash<coll_t> {
+    size_t operator()(const coll_t &c) const { 
+      static rjhash<uint32_t> H;
+      static rjhash<uint64_t> I;
+      return H(c.pgid) ^ I(c.snap);
+    }
+  };
+}
+
 
 
 
@@ -569,8 +667,8 @@ inline ostream& operator<<(ostream& out, const pg_pool_t& p) {
   case CEPH_PG_TYPE_REP: out << "rep"; break;
   default: out << "type " << p.get_type();
   }
-  out << " size " << p.get_size()
-      << " ruleset " << p.get_crush_ruleset()
+  out << " pg_size " << p.get_size()
+      << " crush_ruleset " << p.get_crush_ruleset()
       << " pg_num " << p.get_pg_num()
       << " pgp_num " << p.get_pgp_num()
       << " lpg_num " << p.get_lpg_num()
@@ -1124,31 +1222,31 @@ inline ostream& operator<<(ostream& out, const OSDOp& op) {
     case CEPH_OSD_OP_DELETE:
       break;
     case CEPH_OSD_OP_TRUNCATE:
-      out << " " << op.op.offset;
+      out << " " << op.op.extent.offset;
       break;
     case CEPH_OSD_OP_SETTRUNC:
     case CEPH_OSD_OP_MASKTRUNC:
     case CEPH_OSD_OP_TRIMTRUNC:
-      out << " " << op.op.truncate_seq << "@" << op.op.truncate_size;      
+      out << " " << op.op.trunc.truncate_seq << "@" << op.op.trunc.truncate_size;
       break;
     default:
-      out << " " << op.op.offset << "~" << op.op.length;
+      out << " " << op.op.extent.offset << "~" << op.op.extent.length;
     }
   } else if (ceph_osd_op_type_attr(op.op.op)) {
     // xattr name
-    if (op.op.name_len && op.data.length()) {
+    if (op.op.xattr.name_len && op.data.length()) {
       out << " ";
-      op.data.write(0, op.op.name_len, out);
+      op.data.write(0, op.op.xattr.name_len, out);
     }
-    if (op.op.value_len)
-      out << " (" << op.op.value_len << ")";
+    if (op.op.xattr.value_len)
+      out << " (" << op.op.xattr.value_len << ")";
   } else if (ceph_osd_op_type_exec(op.op.op)) {
     // class.method
-    if (op.op.class_len && op.data.length()) {
+    if (op.op.cls.class_len && op.data.length()) {
       out << " ";
-      op.data.write(0, op.op.class_len, out);
+      op.data.write(0, op.op.cls.class_len, out);
       out << ".";
-      op.data.write(op.op.class_len, op.op.method_len, out);
+      op.data.write(op.op.cls.class_len, op.op.cls.method_len, out);
     }
   }
   return out;
