@@ -243,7 +243,7 @@ public:
   __s64 get_expire_pos() const { return expire_pos; }
   __s64 get_trimmed_pos() const { return trimmed_pos; }
 
-  __s64 get_layout_period() const { return ceph_file_layout_period(layout); }
+  __s64 get_layout_period() const { return layout.fl_stripe_count * layout.fl_object_size; }
   ceph_file_layout& get_layout() { return layout; }
 
   // write
